@@ -1,10 +1,11 @@
 import json
 import os
 import sys
+import time
 import urllib.parse
-from datetime import date, datetime, timedelta
+from datetime import date, datetime
 from re import match, search
-from typing import Any, Dict, List, Optional, Set, Tuple
+from typing import Any, Dict, List, Optional, Set, Tuple, Union
 
 from locust.clients import HttpSession, ResponseContextManager
 from requests import Response
@@ -158,7 +159,7 @@ class _Interactor:
                 self.write_response_to_lib_folder(label, resp)
             return resp
 
-    def login(self, auth: list = None) -> Tuple[HttpSession, Response]:
+    def login(self, auth: list = None, retry: bool = True, check_login: bool = True) -> Tuple[HttpSession, Response]:
         if auth is not None:
             self.auth = auth
         """
@@ -193,17 +194,23 @@ class _Interactor:
         }
         login_url = uri + "auth?appian_environment=tempo"
         log.info(f"Logging in at {self.replace_base_path_if_appropriate(login_url)}")
+        log.info(f"Logging in with user {payload['un']}")
         # Post Auth
         resp = self.post_page(
             login_url,
             payload=urllib.parse.urlencode(payload),
             headers=headers,
             label="Login.SubmitAuth",
-            check_login=False)
+            check_login=check_login)
         if not resp or not resp.ok:
             raise BadCredentialsException()
         elif "__appianMultipartCsrfToken" not in self.client.cookies:
-            raise MissingCsrfTokenException(self.client.cookies)
+            if retry:
+                log.info("__appianMultipartCsrfToken not found, retrying login")
+                time.sleep(1)
+                return self.login(auth, retry=False)
+            else:
+                raise MissingCsrfTokenException(self.client.cookies)
         return self.client, resp
 
     def check_login(self, resp: ResponseContextManager) -> None:
@@ -760,9 +767,13 @@ class _Interactor:
         tabs_list = tab_group_component["tabs"]
         tab_index = 0
         for index, tab in enumerate(tabs_list):
-            if isinstance(tab, dict) and find_component_by_attribute_in_dict("label", tab_label, tab) is not None:
+            try:
+                find_component_by_attribute_in_dict("label", tab_label, tab)
                 tab_index = index + 1
                 break
+            except ComponentNotFoundException:
+                # These are expected when searching through the tabs to find the one containing the label with value tab_label
+                pass
 
         if tab_index:
             new_value = {
@@ -819,27 +830,34 @@ class _Interactor:
         return resp.json()
 
     def upload_document_to_field(self, post_url: str, upload_field: Dict[str, Any],
-                                 context: Dict[str, Any], uuid: str, doc_id: int,
+                                 context: Dict[str, Any], uuid: str, doc_id: Union[int, List[int]],
                                  locust_label: str = None, client_mode: str = 'DESIGN') -> Dict[str, Any]:
         '''
-            Calls the post operation to send an update to a upload_field to upload a document.
-            Requires a previously uploaded document id
+            Calls the post operation to send an update to a upload_field to upload a document or list thereof.
+            Requires a previously uploaded document id or ids
 
             Args:
                 post_url: the url (not including the host and domain) to post to
                 upload_field: the JSON representing the desired checkbox
                 context: the Sail context parsed from the json response
                 uuid: the uuid parsed from the json response
-                doc_id: document id for the upload
+                doc_id: document id or list of document ids for the upload
                 context_label: the label to be displayed by locust for this action
                 client_mode: where this is being uploaded to, defaults to DESIGN
 
             Returns: the response of post operation as json
         '''
-        new_value = {
-            "#t": "CollaborationDocument",
-            "id": doc_id
-        }
+        new_value: Union[List[Dict], Dict]
+        if isinstance(doc_id, int):
+            new_value = {
+                "#t": "CollaborationDocument",
+                "id": doc_id
+            }
+        elif isinstance(doc_id, List):
+            new_value = [{"#t": "CollaborationDocument", "id": id} for id in doc_id]
+        else:
+            log_locust_error(Exception(f"Bad document id or list of document ids: {doc_id}"))
+            sys.exit(2)
         payload = save_builder() \
             .component(upload_field) \
             .context(context) \
