@@ -10,11 +10,14 @@ from ._base import _Base
 from ._interactor import _Interactor
 from .helper import format_label
 from .records_helper import (get_all_records_from_json,
-                             get_record_summary_view_response)
+                             get_record_summary_view_response, get_records_from_json_by_column)
 from .uiform import SailUiForm
 
 log = logger.getLogger(__name__)
 
+RECORDS_INTERFACE_PATH = "/suite/rest/a/sites/latest/D6JMim/pages/records/interface"
+RECORDS_NAV_PATH = "/suite/rest/a/sites/latest/D6JMim/page/records/nav"
+
 
 class _Records(_Base):
     def __init__(self, interactor: _Interactor) -> None:
@@ -36,7 +39,23 @@ class _Records(_Base):
         self._records: Dict[str, Any] = dict()
         self._errors: int = 0
 
-    def get_all(self, search_string: str = None, locust_request_label: str = None) -> Dict[str, Any]:
+    def get_record_interface(self, locust_request_label: str = "Records.Interface") -> Dict[str, Any]:
+        path = RECORDS_INTERFACE_PATH
+        headers = self.interactor.setup_sail_headers()
+        resp = self.interactor.get_page(
+            self.interactor.host + path, headers=headers, label=locust_request_label
+        )
+        return resp.json()
+
+    def get_records_nav(self, locust_request_label: str = "Records.Nav") -> Dict[str, Any]:
+        path = RECORDS_NAV_PATH
+        headers = self.interactor.setup_sail_headers()
+        resp = self.interactor.get_page(
+            self.interactor.host + path, headers=headers, label=locust_request_label
+        )
+        return resp.json()
+
+    def get_all(self, search_string: str = None, locust_request_label: str = "") -> Dict[str, Any]:
         """
         Retrieves all available "records types" and "records" and associated metadata from "Appian-Tempo-Records"
 
@@ -45,7 +64,7 @@ class _Records(_Base):
 
         Returns (dict): List of records and associated metadata
         """
-        self.get_all_record_types()
+        self.get_all_record_types(locust_request_label=locust_request_label)
         for record_type in self._record_types:
             try:
                 self.get_all_records_of_record_type(record_type)
@@ -55,7 +74,7 @@ class _Records(_Base):
 
         return self._records
 
-    def get_all_record_types(self) -> Dict[str, Any]:
+    def get_all_record_types(self, locust_request_label: str = "") -> Dict[str, Any]:
         """
         Navigate to Tempo Records Tab and load all metadata for associated list of record types into cache.
 
@@ -67,9 +86,9 @@ class _Records(_Base):
         headers = self.interactor.setup_request_headers()
         headers['X-Appian-Features-Extended'] = 'e4bc'
         headers["Accept"] = "application/vnd.appian.tv.ui+json"
-        response = self.interactor.get_page(uri=uri, headers=headers, label="Records")
+        breadcrumb = locust_request_label or "Records"
+        response = self.interactor.get_page(uri=uri, headers=headers, label=breadcrumb)
         json_response = response.json()
-
         if not(self._is_response_good(response.text)):
             raise(Exception("Unexpected response on Get call of All Records"))
 
@@ -80,12 +99,13 @@ class _Records(_Base):
 
         return self._record_types
 
-    def get_all_records_of_record_type(self, record_type: str) -> Dict[str, Any]:
+    def get_all_records_of_record_type(self, record_type: str, column_index: int = None) -> Dict[str, Any]:
         """
         Navigate to the desired record type and load all metadata for the associated list of record views into cache.
 
         Args:
             record_type (str): Name of record type for which we want to enumerate the record instances.
+            column_index (int, optional): Column index to only fetch record links from a specific column, starts at 0.
 
         Returns (dict): List of records and associated metadata
 
@@ -96,7 +116,10 @@ class _Records(_Base):
 
         json_response, _ = self._record_type_list_request(record_type)
 
-        self._records[record_type], self._errors = get_all_records_from_json(json_response)
+        if column_index is not None:
+            self._records[record_type], self._errors = get_records_from_json_by_column(json_response, column_index)
+        else:
+            self._records[record_type], self._errors = get_all_records_from_json(json_response)
 
         return self._records
 
@@ -187,7 +210,7 @@ class _Records(_Base):
             raise Exception(f"There is no record type with name {record_type} in the system under test (Exact match = {exact_match})")
         return current_record_type
 
-    def visit_record_instance(self, record_type: str = "", record_name: str = "", view_url_stub: str = "", exact_match: bool = True) -> Tuple[Dict[str, Any], str]:
+    def visit_record_instance(self, record_type: str = "", record_name: str = "", view_url_stub: str = "", exact_match: bool = True, locust_request_label: str = "") -> Tuple[Dict[str, Any], str]:
         """
         This function calls the API for the specific record view/instance to get its response data.
 
@@ -198,6 +221,7 @@ class _Records(_Base):
             record_name (str): Name of the record to be called. If not specified, a random record will be selected.
             view_url_stub (str, optional): page/tab to be visited in the record. If not specified, "summary" dashboard will be selected.
             exact_match (bool, optional): Should record type and record name matched exactly as it is or partial match.
+            locust_request_label(str,optional): Label used to identify the request for locust statistics
 
         Returns (tuple): Responses of Record's Get UI call in a dictionary and the request URI as a string.
 
@@ -239,10 +263,10 @@ class _Records(_Base):
             dashboard_val = current_record.get("dashboard")
             view_url_stub = dashboard_val if dashboard_val else "summary"
 
-        label = f'Records.{record_type}.{record_label}.{view_url_stub}'
+        locust_label = locust_request_label or f'Records.{record_type}.{record_label}.{view_url_stub}'
 
         uri = f"/suite/rest/a/sites/latest/{tempo_site_url_stub}/page/records/record/{opaque_id}/view/{view_url_stub}"
-        resp = self.interactor.get_page(uri=uri, headers=headers, label=label)
+        resp = self.interactor.get_page(uri=uri, headers=headers, label=locust_label)
         return resp.json(), uri
 
     # Alias for the above function to allow backwards compatability
