@@ -5,7 +5,7 @@ import json
 import os
 import random
 import warnings
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Union, Optional
 from urllib.parse import quote, urlparse
 
 from appian_locust.records_helper import _is_grid
@@ -17,9 +17,8 @@ from ._locust_error_handler import raises_locust_error
 from ._task_opener import _TaskOpener
 from ._ui_reconciler import UiReconciler
 from .exceptions import ComponentNotFoundException, InvalidComponentException, ChoiceNotFoundException
-from .helper import (extract_all_by_label, find_component_by_attribute_in_dict,
-                     find_component_by_index_in_dict,
-                     find_component_by_label_and_type_dict)
+from .helper import (extract_all_by_label, find_component_by_attribute_and_index_in_dict, find_component_by_attribute_in_dict,
+                     find_component_by_index_in_dict, find_component_by_label_and_type_dict, find_component_by_type_and_attribute_and_index_in_dict)
 from .records_helper import (get_record_header_response,
                              get_record_summary_view_response,
                              get_url_stub_from_record_list_url_path)
@@ -101,21 +100,63 @@ class SailUiForm:
     def __str__(self) -> str:
         return f"self_state={json.dumps(self.state,indent=4)}"
 
-    # TODO: Handle components on a page with the same label
+    @raises_locust_error
+    def fill_field_by_attribute_and_index(self, attribute: str, attribute_value: str, fill_value: str, index: int = 1, locust_request_label: str = "") -> 'SailUiForm':
+        """
+        Selects a Field by "attribute" and its value provided "attribute_value" and an index if more than one Field is found
+        and fills it with text "fill_value"
+
+        Args:
+            attribute(str): Name of the field to fill
+            attribute_value(str): Value for the attribute passed in to this function
+            fill_value(str): Value to fill in the field
+
+        Keyword Args:
+            index(int): Index of the field to fill if more than one match the attribute and attribute_value criteria (default: 1)
+            locust_request_label(str): Label used to identify the request for locust statistics
+
+        Returns (SailUiForm): The latest state of the UiForm
+
+        Examples:
+
+            >>> form.fill_field_by_attribute_and_index("label", "Write a comment", "Hello, Testing")
+            # selects the first Component with the "label" attribute having "Write a comment" value
+            # and fills it with "Hello, Testing"
+
+            >>> form.fill_field_by_attribute_and_index("label", "Write a comment", "Hello, Testing", 2)
+            # selects the second Component with the "label" attribute having "Write a comment" value
+            # and fills it with "Hello, Testing"
+
+        """
+        component = find_component_by_attribute_and_index_in_dict(attribute, attribute_value, index, self.state)
+        self._validate_component_found(component, attribute_value)
+
+        if component.get("#t", "") not in COMPONENTS_THAT_CAN_BE_FILLED:
+            raise Exception(f"The Component with '{attribute}' = '{attribute_value}' is not a component that can be filled")
+
+        reeval_url = self._get_update_url_for_reeval(self.state)
+        locust_label = locust_request_label or f"{self.breadcrumb}.FillTextFieldByAttribute.{attribute}"
+        new_state = self.interactor.fill_textfield(
+            reeval_url, component, fill_value, self.context, self.uuid, label=locust_label)
+        if not new_state:
+            raise Exception(f"No response returned when trying to update the field with '{attribute}' = '{attribute_value}' at index '{index}'")
+
+        return self._reconcile_state(new_state, form_url=reeval_url)
 
     @raises_locust_error
-    def fill_text_field(self, label: str, value: str, is_test_label: bool = False, locust_request_label: str = "") -> 'SailUiForm':
+    def fill_text_field(self, label: str, value: str, is_test_label: bool = False, locust_request_label: str = "", index: int = 1) -> 'SailUiForm':
         """
         Fills a field on the form, if there is one present with the following label (case sensitive)
         Otherwise throws a NotFoundException
 
         Args:
             label(str): Label of the field to fill out
-            value(str): Value to update the label to
-            is_test_label(bool): If you are filling a text field via a test label instead of a label, set this boolean to true
+            value(str): Value to fill the field with
 
         Keyword Args:
+            is_test_label(bool): If you are filling a text field via a test label instead of a label, set this boolean to true
             locust_request_label(str): Label used to identify the request for locust statistics
+            index(int): Index of the field to fill if more than one match the label criteria (default: 1)
 
         Returns (SailUiForm): The latest state of the UiForm
 
@@ -124,19 +165,8 @@ class SailUiForm:
             >>> form.fill_text_field('Title','My New Novel')
 
         """
-        attribute_to_find = 'testLabel' if is_test_label else 'label'
-        component = find_component_by_attribute_in_dict(
-            attribute_to_find, label, self.state)
-        self._validate_component_found(component, label)
-
-        reeval_url = self._get_update_url_for_reeval(self.state)
-        locust_label = locust_request_label or f"{self.breadcrumb}.FillTextField.{label}"
-        new_state = self.interactor.fill_textfield(
-            reeval_url, component, value, self.context, self.uuid, label=locust_label)
-        if not new_state:
-            raise Exception(f"No response returned when trying to update field with label '{label}'")
-
-        return self._reconcile_state(new_state, form_url=reeval_url)
+        attribute_to_find = 'testlabel' if is_test_label else 'label'
+        return self.fill_field_by_attribute_and_index(attribute_to_find, label, value, index, locust_request_label)
 
     @raises_locust_error
     def fill_field_by_index(self, type_of_component: str, index: int, text_to_fill: str, locust_request_label: str = "") -> 'SailUiForm':
@@ -144,7 +174,7 @@ class SailUiForm:
         Selects a Field by its index and fills it with a text value
 
         Args:
-            type_of_component(str): Name of the componen to fill
+            type_of_component(str): Name of the component to fill
             index(int): Index of the field on the page (is it the first one found, or second etc.)
             value(int): Value to fill in the field of type 'type_of_component'
 
@@ -174,18 +204,19 @@ class SailUiForm:
         return self._reconcile_state(new_state, form_url=reeval_url)
 
     @raises_locust_error
-    def fill_field_by_any_attribute(self, attribute: str, value_for_attribute: str, text_to_fill: str, locust_request_label: str = "") -> 'SailUiForm':
+    def fill_field_by_any_attribute(self, attribute: str, value_for_attribute: str, text_to_fill: str, locust_request_label: str = "", index: int = 1) -> 'SailUiForm':
         """
         Selects a Field by "attribute" and its value provided "value_for_attribute"
         and fills it with text "text_to_fill"
 
         Args:
-            attribute(str): Name of the componen to fill
-            value_for_attribute(int): Value for the attribute passed in to this function
-            text_to_fill(int): Value to fill in the field of type 'type_of_component'
+            attribute(str): Name of the component to fill
+            value_for_attribute(str): Value for the attribute passed in to this function
+            text_to_fill(str): Value to fill the field with
 
         Keyword Args:
             locust_request_label(str): Label used to identify the request for locust statistics
+            index(int): Index of the field to fill if more than one match the label criteria (default: 1)
 
         Returns (SailUiForm): The latest state of the UiForm
 
@@ -196,29 +227,7 @@ class SailUiForm:
             # and fills it with "Hello, Testing"
 
         """
-        component = find_component_by_attribute_in_dict(attribute, value_for_attribute, self.state)
-        if component is None:
-            raise Exception(f"No such component found with attribute: '{attribute}' and its value: '{value_for_attribute}''")
-
-        if component.get("#t", "") not in COMPONENTS_THAT_CAN_BE_FILLED:
-            raise Exception(
-                f'''
-                    The Component found with attribute: '{attribute}' and its value: '{value_for_attribute}'
-                    is not a component that can be filled")
-                ''')
-
-        reeval_url = self._get_update_url_for_reeval(self.state)
-        locust_label = locust_request_label or f"{self.breadcrumb}.FillTextFieldByAttribute.{attribute}"
-        new_state = self.interactor.fill_textfield(
-            reeval_url, component, text_to_fill, self.context, self.uuid, label=locust_label)
-        if not new_state:
-            raise Exception(
-                f'''
-                    No response returned when trying to fill: '{text_to_fill}' in the component found with attribute: '{attribute}'
-                    and its value: '{value_for_attribute}' on the current page")
-                ''')
-
-        return self._reconcile_state(new_state, form_url=reeval_url)
+        return self.fill_field_by_attribute_and_index(attribute, value_for_attribute, text_to_fill, index, locust_request_label)
 
     # Aliases for fill_text_field() function
     fill_paragraph_field = fill_text_field
@@ -300,7 +309,7 @@ class SailUiForm:
         return self._reconcile_state(newer_state)
 
     @raises_locust_error
-    def click(self, label: str, is_test_label: bool = False, locust_request_label: str = "") -> 'SailUiForm':
+    def click(self, label: str, is_test_label: bool = False, locust_request_label: str = "", index: int = 1) -> 'SailUiForm':
         """
         Clicks on a component on the form, if there is one present with the following label (case sensitive)
         Otherwise throws a NotFoundException
@@ -315,6 +324,7 @@ class SailUiForm:
 
         Keyword Args:
             locust_request_label(str): Label used to identify the request for locust statistics
+            index(int): Index of the component to click if more than one match the label criteria (default: 1)
 
         Returns (SailUiForm): The latest state of the UiForm
 
@@ -325,10 +335,10 @@ class SailUiForm:
 
         """
         locust_label = locust_request_label or f"{self.breadcrumb}.Click.{label}"
-        return self._click(label, is_test_label=is_test_label, locust_request_label=locust_label)
+        return self._click(label, is_test_label=is_test_label, locust_request_label=locust_label, index=index)
 
     @raises_locust_error
-    def click_button(self, label: str, is_test_label: bool = False, locust_request_label: str = "") -> 'SailUiForm':
+    def click_button(self, label: str, is_test_label: bool = False, locust_request_label: str = "", index: int = 1) -> 'SailUiForm':
         """
         Clicks on a component on the form, if there is one present with the following label (case sensitive)
         Otherwise throws a NotFoundException
@@ -341,6 +351,7 @@ class SailUiForm:
 
         Keyword Args:
             locust_request_label(str): Label used to identify the request for locust statistics
+            index(int): Index of the component to click if more than one match the label criteria (default: 1)
 
         Returns (SailUiForm): The latest state of the UiForm
 
@@ -351,10 +362,10 @@ class SailUiForm:
 
         """
         locust_label = locust_request_label or f"{self.breadcrumb}.ClickButton.{label}"
-        return self._click(label, is_test_label=is_test_label, locust_request_label=locust_label)
+        return self._click(label, is_test_label=is_test_label, locust_request_label=locust_label, index=index)
 
     @raises_locust_error
-    def click_link(self, label: str, is_test_label: bool = False, locust_request_label: str = "") -> 'SailUiForm':
+    def click_link(self, label: str, is_test_label: bool = False, locust_request_label: str = "", index: int = 1) -> 'SailUiForm':
         """
         Clicks on a component on the form, if there is one present with the following label (case sensitive)
         Otherwise throws a NotFoundException
@@ -367,6 +378,7 @@ class SailUiForm:
 
         Keyword Args:
             locust_request_label(str): Label used to identify the request for locust statistics
+            index(int): Index of the component to click if more than one match the label criteria (default: 1)
 
         Returns (SailUiForm): The latest state of the UiForm
 
@@ -376,15 +388,15 @@ class SailUiForm:
 
         """
         locust_label = locust_request_label or f"{self.breadcrumb}.ClickLink.{label}"
-        return self._click(label, is_test_label=is_test_label, locust_request_label=locust_label)
+        return self._click(label, is_test_label=is_test_label, locust_request_label=locust_label, index=index)
 
-    def _click(self, label: str, is_test_label: bool = False, locust_request_label: str = "") -> 'SailUiForm':
+    def _click(self, label: str, is_test_label: bool = False, locust_request_label: str = "", index: int = 1) -> 'SailUiForm':
         """
         Internal function wrapped by various click methods
         """
         attribute_to_find = 'testLabel' if is_test_label else 'label'
 
-        component = find_component_by_attribute_in_dict(attribute_to_find, label, self.state)
+        component = find_component_by_attribute_and_index_in_dict(attribute_to_find, label, index, self.state)
 
         self._validate_component_found(component, label)
 
@@ -439,6 +451,30 @@ class SailUiForm:
         reeval_url = self._get_update_url_for_reeval(new_state)
         return self._reconcile_state(new_state, form_url=reeval_url)
 
+    @raises_locust_error
+    def click_record_link_by_attribute_and_index(self, attribute: str = "", attribute_value: str = "", index: int = 1, locust_request_label: str = "") -> 'SailUiForm':
+        """
+        Click the index'th record link on the form if there is one present with an attribute matching attribute_value
+        If no attribute is provided, the index'th record link is selected from all record links in the form
+        Otherwise throws a ComponentNotFoundException
+
+        Keyword Args:
+            attribute(str): Attribute to check for 'attribute_value' (default: "")
+            attribute_value(str): Attribute value of record link to click (default: "")
+            index(int): Index of record link to click (default: 1)
+            locust_request_label(str): Label used to identify the request for locust statistics
+
+        Returns (SailUiForm): The record form (feed) for the linked record.
+
+        """
+        component_type = 'RecordLink'
+        component = find_component_by_type_and_attribute_and_index_in_dict(self.state, component_type, attribute, attribute_value, index)
+        locust_label = locust_request_label or f"{self.breadcrumb}.ClickRecordLink"
+        reeval_url = self._get_update_url_for_reeval(self.state)
+        new_state = self.interactor.click_record_link(reeval_url, component, self.context, self.uuid,
+                                                      locust_label=locust_label)
+        return self._reconcile_state(new_state, form_url=reeval_url)
+
     @raises_locust_error
     def click_record_link(self, label: str, is_test_label: bool = False, locust_request_label: str = "") -> 'SailUiForm':
         """
@@ -456,14 +492,24 @@ class SailUiForm:
 
         """
         attribute_to_find = 'testLabel' if is_test_label else 'label'
-        component = find_component_by_attribute_in_dict(attribute_to_find, label, self.state)
-        self._validate_component_found(component, label)
+        return self.click_record_link_by_attribute_and_index(attribute=attribute_to_find, attribute_value=label, locust_request_label=locust_request_label)
 
-        reeval_url = self._get_update_url_for_reeval(self.state)
-        locust_label = locust_request_label or f"{self.breadcrumb}.ClickRecordLink.{label}"
-        new_state = self.interactor.click_record_link(reeval_url, component, self.context, self.uuid,
-                                                      locust_label=locust_label)
-        return self._reconcile_state(new_state, form_url=reeval_url)
+    @raises_locust_error
+    def click_record_link_by_index(self, index: int, locust_request_label: str = "") -> 'SailUiForm':
+        """
+        Click the index'th record link on the form
+        Otherwise throws a ComponentNotFoundException
+
+        Args:
+            index(int): Index of the record link to click (1-based)
+
+        Keyword Args:
+            locust_request_label(str): Label used to identify the request for locust statistics
+
+        Returns (SailUiForm): The record form (feed) for the linked record.
+
+        """
+        return self.click_record_link_by_attribute_and_index(index=index, locust_request_label=locust_request_label)
 
     @raises_locust_error
     def click_start_process_link(self, label: str, site_name: str, page_name: str, is_mobile: bool = False, locust_request_label: str = "") -> 'SailUiForm':
@@ -694,8 +740,7 @@ class SailUiForm:
             raise ChoiceNotFoundException(f"Choice {choice_label} not found for component {label}, valid choices were {choices}")
 
         index = choices.index(choice_label) + 1  # Appian is _sigh_ one indexed
-
-        locust_label = locust_request_label or f'{self.breadcrumb}.SelectDropdownWithLabel.{choice_label}'
+        locust_label = locust_request_label or f'{self.breadcrumb}.SelectDropdownWithLabel.{label}'
         reeval_url = self._get_update_url_for_reeval(self.state)
 
         # Opting to use this field, rather than self.form_url, because 'sail-application-url' is the same between web and mobile
@@ -879,7 +924,7 @@ class SailUiForm:
         return self._reconcile_state(new_state, form_url=reeval_url)
 
     @raises_locust_error
-    def upload_document_to_upload_field(self, label: str, file_path: str, locust_request_label: str = "") -> 'SailUiForm':
+    def upload_document_to_upload_field(self, label: str, file_path: Union[str, List], locust_request_label: str = "") -> 'SailUiForm':
         """
         Uploads a document to a named upload field
         There are two steps to this which can fail, one is the document upload, the other
@@ -911,13 +956,20 @@ class SailUiForm:
 
         # Check again to see if the wrong component
         if component.get('#t') != 'FileUploadWidget':
-            raise Exception(f"Provided component was not a FileUploadWidget, was instead of type '{component.get('#t')}'")
+            if component.get('#t') == "MultipleFileUploadWidget":
+                print("Selected FileUploadWidget is instead MultipleFileUploadWidget, continuing automatically")
+                return self.upload_documents_to_multiple_file_upload_field(label, file_path, locust_request_label)
+            else:
+                raise Exception(f"Provided component was not a FileUploadWidget, was instead of type '{component.get('#t')}'")
+
+        if type(file_path) != str:
+            raise Exception(f"Provided file_path {file_path} was not a string, was instead of type '{type(file_path)}'")
 
         is_encrypted = component.get("isEncrypted", False)
 
-        if not os.path.exists(file_path):
+        if not os.path.exists(str(file_path)):
             raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), file_path)
-        doc_id = self.interactor.upload_document_to_server(file_path, is_encrypted=is_encrypted)
+        doc_id = self.interactor.upload_document_to_server(str(file_path), is_encrypted=is_encrypted)
         locust_label = locust_request_label or f"{self.breadcrumb}.FileUpload.{label}"
         new_state = self.interactor.upload_document_to_field(
             self.form_url, component, self.context, self.uuid, doc_id=doc_id, locust_label=locust_label)
@@ -926,6 +978,63 @@ class SailUiForm:
                 f"No response returned when trying to upload file to field '{label}'")
         return self._reconcile_state(new_state)
 
+    @raises_locust_error
+    def upload_documents_to_multiple_file_upload_field(self, label: str, file_paths: List[str], locust_request_label: str = "") -> 'SailUiForm':
+        """
+        Uploads multiple documents to a named upload field
+        There are two steps to this which can fail, one is the document uploads, the other
+        is finding the component and applying the update.
+
+        Args:
+            label(str): Label of the upload field
+            file_paths(list): List of document file paths in string form
+
+        Keyword Args:
+            locust_request_label(str): Label used to identify the request for locust statistics
+
+        Returns (SailUiForm): The latest state of the UiForm
+
+        Example:
+
+            >>> form.multi_upload_document_to_upload_field('Upload Files', ["/usr/local/appian/File1.zip", "/usr/local/appian/File2.zip"])
+
+        """
+        component = find_component_by_attribute_in_dict(
+            'label', label, self.state)
+
+        self._validate_component_found(component, label)
+
+        # Ensure that the files to upload are actually in a list (as opposed to a string)
+        # we do have type checking, but it apparently allows for strings as a special case of list
+        if type(file_paths) == str:
+            file_paths = list(file_paths)
+
+        # Inner component can be the upload field
+        if component.get('#t') != 'MultipleFileUploadWidget' and 'contents' in component:
+            component = component['contents']
+
+        # Check again to see if the wrong component
+        if component.get('#t') != 'MultipleFileUploadWidget':
+            if component.get('#t') == "FileUploadWidget":
+                print("Selected MultipleFileUploadWidget is instead FileUploadWidget, continuing automatically")
+                return self.upload_document_to_upload_field(label, file_paths[0], locust_request_label)
+            raise Exception(f"Provided component was not a MultipleFileUploadWidget, was instead of type '{component.get('#t')}'")
+
+        is_encrypted = component.get("isEncrypted", False)
+
+        doc_ids: List[int] = []
+        for file_path in file_paths:
+            if not os.path.exists(file_path):
+                raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), file_path)
+            doc_ids.append(self.interactor.upload_document_to_server(file_path, is_encrypted=is_encrypted))
+        locust_label = locust_request_label or f"{self.breadcrumb}.MultiFileUpload.{label}"
+        new_state = self.interactor.upload_document_to_field(
+            self.form_url, component, self.context, self.uuid, doc_id=doc_ids, locust_label=locust_label)
+        if not new_state:
+            raise Exception(
+                f"No response returned when trying to upload file(s) to field '{label}'")
+        return self._reconcile_state(new_state)
+
     @raises_locust_error
     def fill_date_field(self, label: str, date_input: datetime.date, locust_request_label: str = "") -> 'SailUiForm':
         """
